;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
ORG 4084

@DFile=400C
@OldGen=3D00
@NewGen=3C00
@Density=3C20			;POKE density here, 15392

;=======================================
;Generate random number between 0 and 255
;From "Mastering Machine Code on Your ZX81"
;by Toni Baker, 
;=======================================
@RNDNum
	LD HL,(4032)		;(SEED) 
	LD D,H		
	LD E,L
	ADD HL,HL
	ADD HL,HL
	ADD HL,DE
	ADD HL,HL
	ADD HL,HL
	ADD HL,HL
	ADD HL,DE
	LD (4032),HL		;The new random number seed is stored, (SEED) 
	LD A,H			;A=0 to 255
	AND A			;Clear CY
	RET
;=======================================
;Pseudo-randomly fill generation 0 buffer 
;using selected density
;=======================================
@RandGen
	LD HL,OldGen		;OldGen buffer
	LD B,20			;32 cells to check
	LD A,(Density)		;Get the density
	LD C,A
@RGLoop1
	PUSH HL			;PUSH buffer address
	CALL RNDNum		;Random number between 1 and 255
	RR A			;Divide by 2
	POP HL			;POP buffer address
	CP C			;Compare to density	
	JR NC RGLoop2		;IF A>Density THEN GOTO Fill2
	LD (HL),9C		;Cell character, Blackspace
	JR RGLoop3		;Do next cell
@RGLoop2
	LD (HL),00		;Whitespace
@RGLoop3
	INC HL
	DJNZ RGLoop1
	LD DE,OldGen		;Copy results to B$
	JR CopyBS
;=======================================
;1st Copy edge conditions
;  (3D00)->3D20
;  (3D01)->3D21
;  (3D1E)->3CFE
;  (3D1F)->3CFF
;
;Then calculate next generation
;
;   Rule 1: Birth
;      If Cell=OFF and 2 or 3 neighbors
;   Rule 2: Survival
;      If Cell=ON and 2 or 4 neighbors
;   Rule 3: Death
;      If Cell=ON and 0, 1 or 3 neighbors
;=======================================
@NextGen
	LD HL,OldGen
	LD A,(HL)		;Copy left two cells to right edge
	LD (3D20),A
	LD A,(3D01)
	LD (3D21),A
	LD A,(3D1E)		;Copy right two cells to left edge
	LD (3CFE),A
	LD A,(3D1F)
	LD (3CFF),A
@NextCell
	XOR A			;A=0
	LD D,A			;Set counter to 0
	LD E,L			;Save test cell location
	DEC HL			;Start at left 2 neighbors
	DEC HL
	LD B,05			;5 cells to test
@CellLoop
	CP (HL)			;IF (HL)=0 THEN GOTO WhiteCell
	JR Z WhiteCell
	INC D			;Increase counter
@WhiteCell	
	INC HL			;Next cell
	DJNZ CellLoop
	LD L,E			;Restore test cell location
	CP (HL)			;IF (HL)=0 THEN GOTO BirthDeath
	JR Z CellSort
	DEC D			;The test cell is not included in the count
@CellSort
	LD A,(HL)		;Get cell status
	AND A
	LD A,D			;A=Cell count
	JR NZ CellOn		;IF "ON" THEN GOTO CellOn
	CP 02			;IF OFF and cells=2 THEN Birth
	JR Z Birth
	CP 03			;IF OFF and cells=3 THEN Birth
	JR Z Birth
	JR Death		;ELSE No change
@CellOn
	AND A			;IF ON and cells=0 THEN Death
	JR Z Death
	CP 01			;IF ON and cells=1 THEN Death
	JR Z Death
	CP 03			;IF ON and cells=3 THEN Death
	JR Z Death
	JR Birth		;No change
@Death
	XOR A			;Whitespace (Death)
	JR NewCell
@Birth
	LD A,9C			;Blackspace (Birth)
@NewCell
	DEC H			;Set NewGen cell
	LD (HL),A
	INC H			;Back to OldGen and
	INC L			;Next cell right
	LD A,20			;End of buffer?
	CP L
	JR NZ NextCell		;IF NZ THEN do next cell
	LD HL,NewGen		;Copy NewGen to OldGen
	LD DE,OldGen
	LD BC,0020
	LDIR
	LD DE,NewGen		;Copy NewGen to B$
@CopyBS
	LD HL,(4010)		;(VARS)
	LD C,03
	ADD HL,BC		;Start of B$ data section
	EX DE,HL		;Swap for LDIR
	LD C,20			;LDIR byte count
	LDIR			;Copy to B$
	RET
;============================
;Install Border patterns
;============================
	LD HL,Borders		;Border patterns
	LD DE,2200		;Destination font area
	LD BC,0058		;Bytes to move
	LDIR			;Move border patterns
;============================
;Install Cell patterns
;============================
	LD HL,Cells		;Cell patterns
	LD DE,22D8		;Destination font area
	LD C,58			;Bytes to move
	LDIR			;Move cell patterns
	RET
;================================
;Keyboard scan
;================================
@ScanKeys
	CALL 02BB		;"KEYBOARD SCANNING" routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	JR Z NoKey		;No keypress
	CALL 07BD		;"KEYBOARD DECODE" routine, convert BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
@Nokey
	RET
;========================================
;Restore uppercase graphic patterns
;========================================
@UpperPixels
	LD HL,2000		;Lowercase graphic patterns start addr
	LD DE,2200		;Uppercase graphic patterns start addr
	LD B,58			;Counter, 88 byts to transfer
@CPBytes
	LD A,(HL)		;Get pattern
	CPL			;Compliment
	LD (DE),A		;Load pattern
	INC DE			;Increase pointers
	INC HL
	DJNZ CPBytes		;Decrease counter and loop
	LD C,00			;Return 0 to BASIC
	RET
;========================================
;Invert display
;========================================
@Invert
	LD HL,2000
@InvLoop
	LD A,(HL)
	CPL
	LD (HL),A
	INC HL
	LD A,H
	CP 24
	JR NZ InvLoop
	LD BC,0000
	RET
;========================================
;Scroll up then check for key press.
;========================================
@KeySort
	CALL GenXX
	CALL ScanKeys		;Get key pressed
	CP 01			
	JR NZ KeyQ		;IF key pressed then see which one
	LD BC,+10		;GOTO 10
	RET
@KeyQ
	CP "Q"			;(Q)uit?
	JR NZ KeyC
	LD BC,+200		;GOTO 200
	RET
@KeyC
	CP "N"			;(N)ext cell pattern?
	JR NZ KeyP
	LD BC,+300		;GOTO 300
	RET
@KeyP
	CP "P"			;(P)revious cell pattern?
	JR NZ KeyI
	LD BC,+400		;GOTO 400
	RET
@KeyI
	CP "I"			;(I)nvert video display?
	JR NZ KeyS
	LD BC,+500		;GOTO 500
	RET
@KeyS
	CP "S"			;(S)croll pause?
	JR NZ KeyR
	LD BC,+600		;GOTO 600
	RET
@KeyR
	CP "R"			;Restart?
	JR NZ Key6
	LD BC,+700		;GOTO 700
	RET
@Key6
	CP "6"			;Slower?
	JR NZ Key7
	LD BC,+800		;GOTO 800
	RET
@Key7
	CP "7"			;Faster?
	JR NZ KeyRand
	LD BC,+900		;GOTO 900
	RET
@KeyRand
	LD BC,+100		;Any other key generates new random line
	RET
;========================================
;Print generation XX then scroll up
;========================================
@GenXX
	LD HL,(4010)		;(VARS)
	LD BC, 0022
	XOR A			;Clear carry
	SBC HL,BC
	EX DE,HL		;DE=Destination
	LD HL,(4010)		;(VARS)
	INC HL
	INC HL
	INC HL			;HL=Source
	LD C,20			;LDIR byte count
	LDIR			;Copy B$
@Scroll
	LD HL,(DFILE)
	PUSH HL 
	LD DE,0021 
	ADD HL,DE 
	POP DE 
	LD BC,02F7 
	LDIR 
	LD DE,0021 
	SBC HL,DE 
	LD B,20 
@Erase
	INC HL 
	LD (HL),00 
	DJNZ Erase
@Delay
	LD BC,0101		;Scroll delay, POKE HERE
@DLay
	DEC BC			;Decrease counter
	LD A,B
	OR C 
	JR NZ DLay
	RET
;========================================
;Border patterns
;========================================
@Borders
	50 50 50 50 50 50 50 50
	0A 0A 0A 0A FA 02 FE 00
	50 50 50 50 5F 40 7F 00
	00 00 00 00 FF 00 FF 00
	00 FE 02 FA 0A 0A 0A 0A
	0A 0A 0A 0A 0A 0A 0A 0A
	50 50 5F 40 5F 50 50 50
	0A 0A FA 02 FA 0A 0A 0A
	00 7F 40 5F 50 50 50 50
	00 FF 00 FF 00 00 00 00
	00 00 FF 00 FF 00 00 00
;========================================
;Cell patterns
;========================================
@Cells
	7F 7F 7F 00 F7 F7 F7 00
	55 FF 77 EB DD BE 77 FF
	8E DD E8 70 B8 DD 8B 07
	BE C1 F7 F7 EB 1C 7F 7F
	A5 24 E7 00 00 E7 24 A5
	EF BF 1D BF FB D1 FB FE
	EF EF D7 BB 7C BB D7 EF
	E7 DB 5A BD BD 5A DB E7
	7E BD C3 EF F7 C3 BD 7E
	22 55 88 55 22 55 88 55
	18 66 66 81 81 60 60 18
