;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@DFile=400C			;D-File
@FontNo=407B			;FONT Number (SPARE2)
@List=9800			;List of FONT Names

;================================
;Jump Table
;================================
@JumpTable
	JP Name			;Print a FONT Name to List Box	   16516
	JP ListUp		;Scroll List Box up		   16519
	JP ListDown		;Scroll List Box down		   16522
	JP ASCII		;Convert ASCII text to ZX81 format 16525
	JP AtoZ			;Locate FONT A to Z in List Box	   16528
	JP Keys			;Sort out key presses		   16531
	JP KeyUp		;Wait for key to be released	   16534 
	JP Wait			;Pause until key pressed	   16537
	JR FilList		;Fill List Box with names	   16540
	JR REM2			;Initialize program	   	   16542
	JP OptionS		;Save Option screen	 	   16544
	JP OptionR		;Restore Option screen		   16547
	JP UDGMove		;Move UDG routines to 9216	   16550
        JP Swap			;Swap lowercase/uppercase alpha	   16653
	JP Bank1		;Invert Bank 1			   16656
	JP Bank2		;Invert Bank 2			   16659
	00 00 00
;================================
;Fill List Box with names	
;================================
@FilList		
	LD A,01			;FONT #
	LD (FontNo),A		;Store FONT #
	CALL Fill		;Fill the List Box starting with FontNo
	LD A,01			;Reset
	LD (FontNo),A		;FONT #
	RET
;================================
;Print font sample screen
;================================
@SamplePrint
	CALL 2404		;Turn UDGs on
	CALL OptionS		;Save options screen
	CALL Sample		;Print sample screen
	CALL Wait		;Wait for keypress
	CALL OptionR		;Restore options screen
	JP UDG1E		;Turn UDGs off
;================================
;Initialize program
;  Set data pointer
;  Clear memory buffers
;  Load options screen
;================================
@REMData
	00 00
@REM2
	LD HL,0002		;Point to start of 2 REM
	CALL 09D8		;CALL LADDR
	LD BC,000D		;Skip header
	ADD HL,BC
	LD (REMData),HL		;Store address
	LD BC,1FFF		;Clear 32-40K area
	LD HL,8000
	XOR A
	LD (HL),A 
	LD D,H 
	LD E,L 
	INC DE 
	LDIR 
	JP OptionR
;================================
;ASCII to ZX81 conversion
;Convert ASCII Font list to ZX81 format
;================================
@ASCII
	LD HL,1FFF		;Start of ASCII data-1
@ASC01
	INC HL
	LD A,(HL)
	CP 2A			;EOF MARKER (*)
	JR NZ LineFeed
	LD (HL),40		;New EOF Marker
	JR FileStor
@LineFeed
	CP 0D			;Line feed
	JR NZ Numeral
	LD (HL),">"		;Replace ASCII EOL char.
	INC HL
	LD (HL),">"		;Replace ASCII RETURN char.
	JR ASC01
@Numeral			;0 to 9
	CP 30			;ASCII code<0
	JR C Symbol
	CP 3A			;ASCII code>9
	JR NC Upper
	SUB 14
	JR Change
@Upper				;A to Z
	CP 41			;ASCII code<A
	JR C Lower
	CP 5B
	JR NC Lower		;ASCII code>Z
	SUB 1B
	JR Change
@Lower				;a to z
	CP 61			;ASCII code<a
	JR C NoGood
	CP 7B			;ASCII code>z
	JR NC NoGood
	SUB 3B
	JR Change
@Symbol				;Symbols
	EX DE,HL		;Save char address
	LD BC,000F		;Characters to compare
	LD HL,Symbolz
	CPIR			;Locate A in table
	LD A,(HL)		;Get the conversion value
	EX DE,HL		;Restore chr address
	JR Z Change
@NoGood
	LD A,8F			;Unsupported ASCII character
@Change
	LD (HL),A	
	JR ASC01
@Symbolz
	28 10 29 11 3D 14 2B 15
	2D 16 47 3B 19 2C 1A
;================================
;Move file list to DATA storage area
;================================
@FileStor
	LD HL,List		;FONT names list
	PUSH HL			;PUSH destination address
	LD BC,07F8		;Clear the storage area, 255*8=2040 
	LD (HL),00
	LD DE,List+1
	LDIR
	POP DE			;DE=Destination-1
	LD HL,1FFF		;HL=Source-1
@SetB
	LD B,08			;B=Char. counter
@NextCHR
	INC HL			;Source pointer
	LD A,(HL)		;Character to move
	CP 40			;EOF Marker
	JR NZ EOL
	LD (DE),A		;Set EOF Marker
	JR Count
@EOL
	CP ">"			;EOL Markers
	JR NZ Store
	INC HL			;Skip next ">" character
	LD A,08			;Test name length
	CP B			;Length of name
	JR NC Pad		;IF CY THEN name>8 characters
	RST 08			;Stop with error report "N"ame too long
	16
@Pad
	LD A,B			;IF B=0 THEN Loop
	AND A
	JR Z SetB
	INC DE			;Pad name with B spaces
	DJNZ Pad
	JR SetB
@Store
	LD (DE),A		;Move character destination
	INC DE			;DE=next address to load
	DEC B			;Decrease character counter
	JR NextCHR
;================================
;Get FONT file count
;================================
@Count
	LD BC,0001		;Set file counter to 1
	LD HL,List		;FONT names list
	LD DE,0008		;Bytes/name
@Count01
	ADD HL,DE		;Skip to next name
	BIT 6,(HL)		;EOF marker?
	JR Z Count02
	PUSH BC			;Push file count
	CALL Options		;Copy nav screen to DFile
	CALL FilList		;Fill list box with font names
	POP BC			;Pop file count
	RET
@Count02
	INC C			;Increase file counter
	JR Count01
;================================
;Print FONT file name in List Box
;================================
@Name				;Locate FONT name addr
	LD HL,List-8		;FONT names list
	LD DE,0008		;Bytes/name
	LD A,(FontNo)		;FONT # to find
	LD B,A			;
@Font01
	ADD HL,DE		;Move pointer
	DJNZ Font01
	PUSH HL			;PUSH source addr
	LD HL,(DFile)		;Locate print position in D-FILE
	INC HL			;Add column offset
	INC HL
	INC HL
	LD A,(407C)		;(ROW), add row offset
	LD B,A
	LD E,21			;Bytes/Line
@Font02
	ADD HL,DE
	DJNZ Font02
	POP DE			;POP source addr
	EX DE,HL		;DE=destination, HL=source
	LD C,08			;8 Bytes to move
	LDIR			;Move name to D-FILE
	RET
;================================
;Fill LIST BOX
;================================
@Fill
	LD A,02			;ROW #
	LD (407C),A		;(ROW), save row #
	LD B,14			;Lines to fill counter
@Fill01
	PUSH BC			;PUSH counter
	CALL NAME		;Print name
	POP BC			;POP counter
	BIT 6,(HL)		;EOF marker?
	JR Z Fill02
	XOR A			;Roll list back to
	LD (FontNo),A   	;FONT #1 if end reached
@Fill02
	LD HL,FontNo		;FontNo addr
	INC (HL)		;Increase FontNo
	LD HL,407C		;ROW # addr
	INC (HL)		;Increase ROW #
	DJNZ Fill01		;Print next name
	LD C,00
	RET
;================================
;GOTO A to Z in LIST BOX
;================================
@AtoZ
	LD DE,0008
	LD A,(4021)		;(Spare1), Char. to find
	LD B,A			;SAVE char. code in B
	LD C,D			;Set counter to 0
	LD HL,List-8		;Names list-8
@Search
	ADD HL,DE		;1st char. of name
	INC C			;Increase name counter
	CP (HL)			;Match found?
	JR Z Match
	INC HL			;Test for EOF
	BIT 6,(HL)		;EOF marker?
	JR Z NextFont
	LD BC,0000		;RETURN 0 to BASIC
	RET			;IF No match found
@NextFont
	LD A,B			;A=Char. code
	DEC HL
	JR Search		;Check next name
@Match
	LD B,D			;B=0, C=FONT number
	LD A,C
	LD (FilList+1),A
	PUSH BC			;PUSH FONT number
	CALL FilList		;Fill List Box
	POP BC			;RETURN FONT Number to BASIC
	RET
;================================
;Scroll LIST BOX up
;================================
@ListUp
	LD DE,0045		;NW corner,ROW*33+3
	LD HL,(DFile)
	ADD HL,DE		;HL=Source
	LD B,13			;Rows to scroll
@Up
	LD A,B			;Save row counter in A
	LD BC,0021		;Bytes/ROW
	LD D,H			;DE=destination
	LD E,L
	ADD HL,BC		;HL=source addr.
	PUSH HL			;PUSH source
	LD C,08			;8 characters/row
 	LDIR			;Move row up
	POP HL			;POP source
	LD B,A			;B=row counter
	DJNZ Up			;IF NZ THEN move next row
	RET
;================================
;Scroll LIST BOX down
;================================
@ListDown
	LD HL,(DFile)
	LD DE,02BF		;SW corner, Last ROW*33+10
	ADD HL,DE		;HL=Destination addr
	LD B,13			;ROW count
@Down
	LD A,B			;Save row counter in A
	LD BC,0021		;Bytes/ROW
	LD D,H			;DE=destination
	LD E,L		
	AND A			;Clear carry
	SBC HL,BC		;HL=source addr.
	PUSH HL			;PUSH source
	LD C,08			;8 characters/row
 	LDDR			;Move row up
	POP HL			;POP source
	LD B,A			;B=row counter
	DJNZ Down		;IF NZ THEN move next row
	RET
;================================
;Keyboard scan
;================================
@KeyScan
	CALL 02BB		;CALL KEYBOARD SCANNING routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL 07BD		;CALL KEYBOARD DECODE routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
;Wait for keypress
;================================
@Wait
	LD A,(4025)		;Get (LASTK)
	CP FF			;Key down?
	JR Z Wait		;No, wait for keypress
;================================
;Wait for key to be released
;================================
@KeyUp
	LD A,(4025)		;Get (LASTK)
	CP FF			;Key down?
	JR NZ KeyUp		;Yes, wait for key release
	LD BC,0000		;RET 0
	RET
;================================
;Sort the keys
;================================
@Keys
	CALL KeyScan
	CP 01			;No key?
	JR Z Keys		;Yes, loop until key pressed
	AND A			;Space key
	JR Z Keys
	CP "A"			;Code less than "A"?
	JR C Numeric		;Yes, GOTO Numeric
@KeyNL
	CP 76			;Select Font (Key N/L)?
	JR NZ Alpha
	LD BC,+700		;GOTO 700
	RET
@Alpha
	LD (4021),A		;(Spare1), Store character code for Alpha search
	LD BC,+800		;GOTO 800
	RET
@Numeric
	SUB 1C			;Convert code to 0 to 9
	RL A			;A=0,2,4,6,...
	LD HL,GOTO		;GOTO table base address
	LD B,00			;BC=Table offset
	LD C,A
	ADD HL,BC		;GOTO table pointer
	LD B,(HL)		;Return GOTO value to BASIC
	INC HL
	LD C,(HL)
	RET
@GOTO
	03 E8	;Key 0=GOTO 1000
	04 4C	;Key 1=GOTO 1100
	04 B0	;Key 2=GOTO 1200
	05 14	;Key 3=GOTO 1300
	05 78	;Key 4=GOTO 1400
	02 58	;Key 5=GOTO 600
	01 2C	;Key 6=GOTO 300
	01 90	;Key 7=GOTO 400
	01 F4	;Key 8=GOTO 500
	07 6C	;Key 9=GOTO 1900
;================================
;Store/Restore Options screen
;================================
@OptionS
	LD BC,0318		;Byte count
	LD HL,(DFile)
	INC HL			;HL=DFile
	LD DE,(REMData)		;DE=Options screen
	LDIR
	RET
@OptionR
	LD BC,0318		;Byte count
	LD HL,(REMData)		;HL=Options screen
@Muv
	LD DE,(DFile)
	INC DE			;DE=DFile
	LDIR
	RET
;================================
;Display Sample screen
;================================
@Sample
	LD BC,0318		;Byte count
	LD HL,(REMData)
	ADD HL,BC		;HL=Sample screen
	JR Muv
;================================
;Swap Lower/Upper
;================================
@Swap
	LD HL,2130		;Lowercase start
	LD DE,2800		;Store lowercase at 2800
	LD A,D0			;Bytes to move
	LD B,E
	LD C,A			;Bytes to move
	LDIR			;Move lowercase to storage
	LD C,A			;Bytes to move
	LD DE,2130		;Lowercase start
	LD HL,2330		;Uppercase start
	LDIR			;Move uppercase to lowercase
	LD C,A			;Bytes to move
	LD HL,2800		;Stored lowercase start
	LD DE,2330		;Uppercase start
	LDIR			;Move lowercase to uppercase
	LD HL,2130		;Lowercase start
	LD B,D0
@Comp1
	LD A,(HL)
	CPL
	LD (HL),A
	INC HL
	DJNZ Comp1
	LD HL,2330		;Uppercase start
	LD B,D0
@Comp2
	LD A,(HL)
	CPL
	LD (HL),A
	INC HL
	DJNZ Comp2
	RET
;================================
;I Reg vectors
;================================
@UDGMove
	LD DE,2400
	LD HL,UDG1E
	LD BC,000D
	LDIR
	RET
@UDG1E
	LD A,1E
	JR SetI
@UDG20
	LD A,20
	JR SetI
@UDG22
	LD A,22
@SetI
	LD I,A
	RET
;================================
;Invert bank 1 and/or 2
;================================
@Bank1
	LD HL,2000		;Start of Bank 1
	LD C,22			;End of Bank 1
	JR Switch		;Compliment Bank 1
@Bank2
	LD HL,2200		;Start of Bank 2
	LD C,24			;End of Bank 2
@Switch
	LD A,(HL)		;Compliment contents
	CPL
	LD (HL),A		;Replace
	INC HL			;Next byte
	LD A,H			;Test for end of Bank
	CP C
	JR NZ Switch
	LD BC,0000		;Return 0 to basic
	RET