;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================

ORG 449F

@VARS=4010
@FONT=2000			;Font bit patterns

	JR MuvRem
	JR Muv8K
	JR Numbers
	JR Bank2
	JR Swap
	JR Invert1
	JR Invert2
	JP UDGs
	00 00 00
	00 00 00
	00 00 00

;================================
;Store font and UDGs in 1 REM
;================================
@MuvRem
	LD HL,Font		;Font location
	LD DE,408A		;Destination, 1 REM line
	LD BC,0400		;1024 Bytes to move
	LDIR			;Move font data to 1 REM line
	RET
;================================
;Install Font & I reg vector calls
;================================
@Muv8K
	LD DE,Font		;Font location
	LD HL,408A		;1 REM line
	LD BC,040D		;1037 Bytes to move
	LDIR			;Move font data to $2000
	RET
;================================
;Restore number patterns
;================================
@Numbers
	LD DE,22D8		;Uppercase "0" location
	LD HL,20D8		;Lowercase "0" location
	LD BC,0058		;Bytes to move
	LDIR			;Move number data to uppercase
	RET
;=========================
; Make Bank 2
; If bank 2 is missing this routine will replace it with a duplicate of bank 1.
;=========================
@Bank2
	LD HL,2000		;Copy Bank 1 to Bank 2
	LD DE,2200
	LD BC,0200
	LDIR
	RET
;================================
;Swap Lower/Upper
;================================
@Swap
	LD HL,2130		;Lowercase start
	LD DE,2800		;Store lowercase at 2800
	LD A,D0			;Bytes to move
	LD B,E
	LD C,A			;Bytes to move
	LDIR			;Move lowercase to storage
	LD C,A			;Bytes to move
	LD DE,2130		;Lowercase start
	LD HL,2330		;Uppercase start
	LDIR			;Move uppercase to lowercase
	LD C,A			;Bytes to move
	LD HL,2800		;Stored lowercase start
	LD DE,2330		;Uppercase start
	LDIR			;Move lowercase to uppercase
	LD HL,2130		;Lowercase start
	CALL Comp
	LD HL,2330		;Uppercase start
@Comp
	LD B,D0
@Comp1
	LD A,(HL)
	CPL
	LD (HL),A
	INC HL
	DJNZ Comp1
	RET
;================================
;Invert bank 1 and/or 2
;================================
@Invert1
	LD HL,2000		;Start of Bank 1
	LD C,22			;End of Bank 1
	JR Switch		;Compliment Bank 1
@Invert2
	LD HL,2200		;Start of Bank 2
	LD C,24			;End of Bank 2
@Switch
	LD A,(HL)		;Compliment contents
	CPL
	LD (HL),A		;Replace next byte
	INC HL
	LD A,H			;Test for end of Bank
	CP C
	JR NZ Switch
	RET
;==============================================================================
;Install custom characters
;Convert binary data to hex, then replace pattern in font table.
;A$=38+32=70 (String Var Code=Chr Code + 32)
;A$ contains the data string to process.  The 1st character is the character to
;replace followed by the bit patterns.
;==============================================================================
@UDGs
	LD C,46			;C=A$ CODE
	LD DE,(VARS)		;HL=start of VARS
@NextVar
	EX DE,HL
	LD A,(HL) 		;Find A$
	CP C			;A$?
	JR Z AString 		;A$ FOUND
	CP 80 			;End of VARS?
	JP Z 0D4B		;REPORT-2, Variable (A$) not found
	PUSH BC			;Save A$ code
	CALL 09F2		;The "Next line or variable' subroutine
	POP BC			;C=A$ CODE 
	JR NextVar
@AString
	INC HL			;Skip over A$ header info
	INC HL
	INC HL			;HL=Start of A$ text section
	LD A,(HL)		;Character code to replace with new patterns
	CP 40 			;Graphic chr?
	JR C Normal		;No
	SBC A,40 		;Subtract 64 from character code
@Normal
	EX DE,HL		;DE=Start of A$ Data
	INC DE			;DE=Start of "bit pattern" data
	LD C,A			;Locate pattern in font table
	XOR A			;Clear Carry 
	LD B,A
	RL C 			;Code*2
	RL B
	RL C	 		;Code*4
	RL B 
	RL C 			;Code*8
	RL B
	LD HL,Font		;Pattern table base address
	ADD HL,BC		;HL=$2000 + Chr. Code*8
@LoopA
	LD B,08 		;Byte counter
	XOR A			;A=0
	LD (HL),A		;Reset all bits
@LoopB
	LD A,(DE)		;A=A$ data
	CP "1"			;"1" character?
	JR Z SetBit		;Yes, set this bit
	CP 80			;Black space?
	JR NZ NoSet		;Yes, set this bit
@SetBit
	LD C,B			;C=B
	RL C			;C=B*2
	RL C			;C=B*4
	RL C			;C=B*8
	LD A,BE			
	ADD A,C			;A=Opcode value
	LD (BitSet+1),A		;Self modifying code change opcode value
				;depending on bit being tested.
@BitSet
	SET 7,(HL)		;SET [BE+B*8],(HL) Opcode
@NoSet
	INC DE			;Next Data address
	DJNZ LoopB
	INC HL			;Next Font address
	LD A,(DE)		;A=A$ data
	CP FF			;End of data marker?
	RET Z
	JR LoopA